########################################################
# Copyright © 2009 Six Apart, Ltd.

package HTML::Laundry::Rules::Default;
use strict;
use warnings;
use base qw( HTML::Laundry::Rules );

=head1 NAME

HTML::Laundry::Rules::Default - follows default behavior in HTML::Laundry::Rules

=cut

1;